"use strict";

var util = require("./util"),
    noLuckyChild;

exports.render = function (ast) {
  var buffer = "",
      indent = -1,
      mode = "line",
      renderSpan = function renderSpan(text) {
    undefined;
    buffer = buffer + text;
  },
      renderNewline = function renderNewline() {
    buffer = util.trimRightSpace(buffer);
    renderSpan("\n");
    renderSpan(util.spaces(indent * 2));
  },
      increaseIndent = function increaseIndent() {
    indent = indent + 1;
  },
      decreaseIndent = function decreaseIndent() {
    indent = indent - 1;
  };

  var render = (function (_render) {
    function render(_x, _x2, _x3, _x4, _x5, _x6) {
      return _render.apply(this, arguments);
    }

    render.toString = function () {
      return _render.toString();
    };

    return render;
  })(function (node, parent, index, inline, lucky, afterDollar) {
    undefined;

    switch (util.type(node)) {
      case "string":
        switch (mode) {
          case "line":
            renderNewline();
            renderSpan(", ");
            renderSpan(util.markToken(node));
            mode = "text";
            break;

          case "text":
            renderSpan(" ");
            renderSpan(util.markToken(node));
            break;

          case "start":
            renderSpan(util.markToken(node));
            mode = "text";
            break;
        }

        break;

      case "array":
        switch (mode) {
          case "line":
            renderNewline();

            if (util.isDeep(parent[index - 1])) {
              renderNewline();
            }

            mode = "start";
            increaseIndent();
            noLuckyChild = true;
            node.forEach(function (child, i) {
              render(child, node, i, i === 0, noLuckyChild, false);

              if (util.isArray(child)) {
                noLuckyChild = false;
              }
            });
            decreaseIndent();
            mode = "line";
            break;

          case "text":
            if (inline) {
              renderSpan(" (");
              mode = "start";
              node.forEach(function (child, i) {
                render(child, node, i, i === 0, true, false);
              });
              renderSpan(")");
              mode = "text";
              return null;
            }

            if (index + 1 === parent.length && !afterDollar) {
              renderSpan(" $");
              noLuckyChild = true;
              node.forEach(function (child, i) {
                render(child, node, i, i === 0, noLuckyChild, true);

                if (util.isArray(child)) {
                  noLuckyChild = false;
                }
              });
              mode = "line";
            } else {
              if (util.isPlain(node) && lucky) {
                renderSpan(" (");
                mode = "start";
                node.forEach(function (child, i) {
                  render(child, node, i, i === 0, true, false);
                });
                renderSpan(")");
                mode = "text";
              } else {
                renderNewline();
                mode = "start";
                increaseIndent();
                noLuckyChild = true;
                node.forEach(function (child, i) {
                  render(child, node, i, i === 0, noLuckyChild, false);

                  if (util.isArray(child)) {
                    noLuckyChild = false;
                  }
                });
                decreaseIndent();
                mode = "line";
              }
            }

            break;

          case "start":
            if (inline && parent !== ast) {
              renderSpan("(");
              mode = "start";
              node.forEach(function (child, i) {
                render(child, node, i, true, false, false);
              });
              renderSpan(")");
              mode = "text";
            } else {
              increaseIndent();
              noLuckyChild = true;
              node.forEach(function (child, i) {
                render(child, node, i, i === 0, true, false);

                if (util.isArray(child)) {
                  noLuckyChild = false;
                }
              });
              decreaseIndent();
              mode = "line";
            }

            break;
        }

        break;
    }

    return buffer + "\n";
  });

  if (!ast.every(util.isArray)) {
    throw new Error("Cirru AST uses nested arrays");
  }

  return render(ast, [], 1, false, false, false);
};